@@ -30,7 +30,7 @@ REPODIR=${REPODIR:-$BASE/new}
 
 # TODO: Figure out how to get this on to the box properly
 sudo apt-get update
-sudo apt-get install -y --force-yes libvirt-dev libxml2-dev libxslt-dev libmysqlclient-dev libpq-dev libnspr4-dev pkg-config libsqlite3-dev libzmq-dev libffi-dev libldap2-dev libsasl2-dev ccache libkrb5-dev uuid-dev swig
+sudo apt-get install -y --force-yes libvirt-dev libxml2-dev libxslt-dev libmysqlclient-dev libpq-dev libnspr4-dev pkg-config libsqlite3-dev libzmq-dev libffi-dev libldap2-dev libsasl2-dev ccache
 
 # FOR numpy / pyyaml
 sudo apt-get build-dep -y --force-yes python-numpy
@@ -148,82 +148,19 @@ $epvenv/bin/test_cmd | grep 'Test cmd'
 
 projectdir=$tmpdir/projects
 mkdir -p $projectdir
+sudo chown -R $USER $REPODIR
 
-requirementsvenv=$tmpdir/update
-mkvenv $requirementsvenv
-$requirementsvenv/bin/python -m pip install $REPODIR/requirements
-
-for PROJECT in $PROJECTS ; do
-    SHORT_PROJECT=$(basename $PROJECT)
-    if ! grep 'pbr' $REPODIR/$SHORT_PROJECT/setup.py >/dev/null 2>&1
-    then
-        # project doesn't use pbr
-        continue
-    fi
-    if [ $SHORT_PROJECT = 'pypi-mirror' ]; then
-        # pypi-mirror doesn't consume the mirror
-        continue
-    fi
-    if [ $SHORT_PROJECT = 'jeepyb' ]; then
-        # pypi-mirror doesn't consume the mirror
-        continue
-    fi
-    if [ $SHORT_PROJECT = 'tempest' ]; then
-        # Tempest doesn't really install
-        continue
-    fi
-    if [ $SHORT_PROJECT = 'requirements' ]; then
-        # requirements doesn't really install
-        continue
-    fi
-    if [ $SHORT_PROJECT = 'keystoneauth-saml2' ]; then
-        # EOL - Repo dead
-        continue
-    fi
-
-    pushd $REPODIR/$SHORT_PROJECT
-    if [ $(git rev-parse --abbrev-ref HEAD) != 'stable/kilo' ]; then
-        continue
-    fi
-    popd
-
-    # set up the project synced with the global requirements
-    sudo chown -R $USER $REPODIR/$SHORT_PROJECT
-    (cd $REPODIR/requirements && $requirementsvenv/bin/python update.py $REPODIR/$SHORT_PROJECT)
-    pushd $REPODIR/$SHORT_PROJECT
-    if ! git diff --quiet ; then
-        git commit -a -m'Update requirements'
-    fi
-    popd
-
-    # Clone from synced repo
-    shortprojectdir=$projectdir/$SHORT_PROJECT
-    git clone $REPODIR/$SHORT_PROJECT $shortprojectdir
-
-    # Test that we can make a tarball from scratch
-    sdistvenv=$tmpdir/sdist
-    mkvenv $sdistvenv
-    cd $shortprojectdir
-    $sdistvenv/bin/python setup.py sdist
-
-    cd $tmpdir
-
-    # Test that the tarball installs
-    tarballvenv=$tmpdir/tarball
-    mkvenv $tarballvenv
-    $tarballvenv/bin/pip $PIPFLAGS install -f $WHEELHOUSE $shortprojectdir/dist/*tar.gz
-
-    # Test pip installing
-    pipvenv=$tmpdir/pip
-    mkvenv $pipvenv
-    $pipvenv/bin/pip $PIPFLAGS install -f $WHEELHOUSE git+file://$shortprojectdir
-    # Ensure the install_package_data is doing the thing it should do
-    if [ $SHORT_PROJECT = 'nova' ]; then
-        find $pipvenv | grep migrate.cfg
-    fi
-
-    # Test pip install -e
-    pipvenv=$tmpdir/pip
-    mkvenv $pipvenv
-    $pipvenv/bin/pip $PIPFLAGS install -f $WHEELHOUSE -e $shortprojectdir
-done
+export PBR_INTEGRATION=1
+export PIPFLAGS
+export PIPVERSION
+PBRVERSION=pbr
+if [ -n "$PBR_CHANGE" ] ; then
+    PBRVERSION=$(ls $pbrsdistdir/dist/pbr-*.whl)
+fi
+export PBRVERSION
+export PROJECTS
+export REPODIR
+export WHEELHOUSE
+export OS_TEST_TIMEOUT=600
+cd $REPODIR/pbr
+tox -epy27 -- test_integration
